/*
 * fasta-char-counter  -  Tool for counting characters in a fasta file.
 *
 * Version 0.1.0 (May 20, 2014)
 *
 * Copyright (c) 2014 Kirill Kryukov
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define PROGRAM_NAME      "FASTA Char Counter"
#define EXECUTABLE_NAME   "fasta-char-counter"
#define VERSION           "0.1.0"
#define RELEASE_DATE      "2014-05-20"
#define COPYRIGHT_YEARS   "2014"
#define COPYRIGHT_HOLDER  "Kirill Kryukov"

#define BUFSIZE 1048576

int main(int argc,char **argv)
{
    FILE *F = 0;
    unsigned long long charcount[256];
    unsigned char buf[BUFSIZE], *pos, *end;
    int show_version = 0, show_help = 0;
    int i;

    memset(charcount,0,sizeof(*charcount)*256);

    char *in_file_name = 0;
    char *charcount_file_name = 0;

    if (argc<2) { show_version = 1; show_help = 1; }
    else
    {
        for (i=1; i<argc; i++)
        {
            if (i<argc-1)
            {
                if (!strcmp(argv[i],"--in")) { in_file_name = argv[++i]; continue; }
                if (!strcmp(argv[i],"--out")) { charcount_file_name = argv[++i]; continue; }
            }
            if (!strcmp(argv[i],"--version")) { show_version = 1; continue; }
            if (!strcmp(argv[i],"--help")) { show_help = 1; continue; }
        }
    }

    if (show_version) { static char *version_str = PROGRAM_NAME ", version " VERSION ", " RELEASE_DATE "\n"
                                                   "Copyright (c) " COPYRIGHT_YEARS " " COPYRIGHT_HOLDER "\n";
                        fputs(version_str,stderr); }
    if (show_help) { fputs("Usage: fasta-char-counter --in <FASTA file> --out <charcount file>\n",stderr); }
    if ((!in_file_name || !charcount_file_name) && (show_help || show_version)) { exit(0); }

    if (!in_file_name) { printf("Error: FASTA file is not specified with '--in' option\n"); exit(1); }
    if (!charcount_file_name) { printf("Error: output file is not specified with '--out' option\n"); exit(1); }

    F = fopen(in_file_name,"rb");
    if (!F) { printf("Can't open input FASTA file \"%s\"\n",in_file_name); exit(1); }

#define REFILL() do { pos = buf; end = buf + fread(buf,1,BUFSIZE,F); } while (0)

    REFILL();
    while (pos < end)
    {
        while (pos < end && *pos!='>') { pos++; }
        if (pos < end) { break; } else { REFILL(); }
    }
    while (pos < end)
    {
        charcount['>']++;
        while (pos < end)
        {
            while (pos < end && *pos!=13 && *pos!=10)  { pos++; }
            if (pos < end) { break; } else { REFILL(); }
        }
        while (pos < end)
        {
            while (pos < end && *pos!='>') { charcount[*(unsigned char*)pos]++; pos++; }
            if (pos < end) { break; } else { REFILL(); }
        }
    }
    fclose(F);

    F = fopen(charcount_file_name,"w");
    if (!F) { printf("Can't create charcount file \"%s\"\n",charcount_file_name); exit(1); }
    for (i=32; i<256; i++) { if (charcount[i]) { fprintf(F,"%c %llu\n",i,charcount[i]); } }
    fclose(F);

    return 0;
}
